<?php
/**
 * Plugin Name: Post Views
 * Description: Records visits to each post
 * Author: Tim Kaye
 * Author URI: https://timkaye.org
 * Version: 0.2
 */

/* REGISTER CUSTOM VIEWS REST API ROUTE */
function kts_register_views_route() {
    register_rest_route( 'kts/v1', '/views', array(
        'methods'  => 'POST',
        'callback' => 'kts_views_callback',
        'permission_callback' => function() {
			return ! current_user_can( 'manage_options' );
		}
    ) );
}
add_action( 'rest_api_init', 'kts_register_views_route' );

/* VIEWS CALLBACK */
function kts_views_callback( $request ) {

	# Bail if views is not 1
	if ( $request['views'] !== 1 ) {
		return;
	}

	# Do not count views by administrators
	if ( current_user_can( 'manage_options' ) ) {
		return;
	}
			
	# Get views count
	$views = (int) get_post_meta( $request['post_id'], 'views', true ); // string cast as integer

	$new_views_count = $views + 1;

	# Increment views count
	update_post_meta( $request['post_id'], 'views', $new_views_count );

	return $new_views_count;
}


/* POSTS MENU ORDER AND META */
function kts_post_views_column( $columns ) {
    $columns['views'] = 'Views';
    return $columns;
}
add_filter( 'manage_post_posts_columns', 'kts_post_views_column' );
add_filter( 'manage_edit-post_sortable_columns', 'kts_post_views_column' );

function kts_render_post_views_column( $column, $post_id ) {
    switch( $column ) {
		case 'views':
			$views = get_post_meta( $post_id, 'views', true );
			echo absint( $views );
        break;
    }
}
add_action( 'manage_post_posts_custom_column', 'kts_render_post_views_column', 10, 2 );


/* ENQUEUE JAVASCRIPT COOKIE */
function kts_enqueue_cookie_script() {
	if ( ! is_singular( 'post' ) ) {
		return;
	}
	wp_enqueue_script( 'js-cookie', plugin_dir_url( __FILE__ ) . 'js/js-cookie.min.js', null, '3.0.5', true );
	wp_enqueue_script( 'count-js', plugin_dir_url( __FILE__ ) . 'js/count.js', array( 'js-cookie' ), '0.2', true );

	# For front-end AJAX and REST API for counting post views
	wp_localize_script( 'count-js', 'VIEWS', array(
		'rest_url'		=> esc_url_raw( rest_url() ),
		'views_nonce'	=> wp_create_nonce( 'wp_rest' ),
		'post_id'		=> get_the_ID(),
	) );
}
add_action( 'wp_enqueue_scripts', 'kts_enqueue_cookie_script' );


/* SHORTCODE FOR MOST POPULAR POSTS */
function kts_popular_posts_shortcode() {
	$posts = get_posts( array(
		'numberposts'	=> 5,
		'post_status'	=> 'publish',
		'order'			=> 'DESC',
		'orderby'		=> 'meta_value_num',
		'meta_key'		=> 'views',
	) );

	$populars = '<h2>Most Popular Posts</h2>';
	$populars .= '<ol>';
	foreach( $posts as $post ) {
		$populars .= '<li><a href="' . esc_url( $post->post_permalink ) . '">' . esc_html( $post->post_title ) . '</a></li>';
	}
	$populars .= '</ol>';

	return $populars;

}
add_shortcode( 'popular_posts', 'kts_popular_posts_shortcode' );
